//include the config file
include <config.scad>


//M O D U L E S: 
//0. Basic Calls - Diese Module werden aus anderen Dateien aufgerufen und nutzen die Library eigenen Module zum erstellen eines Würfels
//Note: f/fs hinter variablen steht für Flag, soll heißen es ist ein boolean/boolean array


//Diese parametergesteuerte Funktion wird in dem Output aus dem Unity Programm aufgerufen 
//Sie erzeugt die verschiedenen Visualisierungen des Qubit Registers
module generate_from_unity(mode, probs=[], measuringfs=[], measured = [], measureResults=[] ){
    if(mode == 1){ //one qubit mode (along x axis visualized)
        visualize1Qbit(probs, measuringfs, measured, measureResults);
    } else if(mode == 2) {//two qubit mode
        visualize2Qbit(probs, measuringfs, measured, measureResults);
    } else if(mode == 3) { //default cube building mode
        visualize3Qbit(mode, probs, measuringfs, measured, measureResults);
    }
}


//Diese Funktion erstellt eine Visualisierung von einem Qbit, entlang der X-Achse(1. Qbit)
module visualize1Qbit(WSK = [], measuringfs = [], measured = [], measureResults = []){
    //Draw Edge   
    drawCylinder([t,-t,-t],[0,-90,0],false);
    
    //Draw Probability Background Squares
    draw1QbitVertices();
    
    //Draw Probability Coloring Squares
    draw1QbitProbabilities(WSK);
    
    //Draw Probability Labels
    draw1QbitProbLabels(WSK);
    
    //Draw State Labels
    draw1QbitStateLabels(1);
    
    //Build Square with edges, cut edges if indicating measurement
    difference(){
        union(){
            draw1QbitEdges();
        }
        if(measuringfs[0] == true){
            drawMeasurementPane(1, 1);
        }   
    }
    
    //draw panes for indicating measurement
    if(measuringfs[0] == true){
        drawMeasurementPane(1, 1);
    }
}


//Diese Funktion erstellt ein Quadrat, welches 2 Qbits visualisiert
module visualize2Qbit(WSK=[], measuringfs=[], measured = [], measureResults=[]){
   
    //Draw Probability Background Squares
    draw2QbitVertices();
    
    //Draw Probability Coloring Squares
    draw2QbitProbabilities(WSK);
    
    //Draw State Labels
    draw2QbitStateLabels(2);
    
    //Draw Probability Coloring Squares
    draw2QbitProbLabels(WSK);
    
    //Draw Probability Labels
    draw2QbitStateLabels(2);
    
    //Build Square with edges, cut edges if indicating measurement
    difference(){
        union(){
            draw2QbitEdges();
        }
        for(idx=[0 : len(measuringfs) - 1]){
            if(measuringfs[idx] == true){
                drawMeasurementPane(idx + 1, 0);
            }
        }
    }
    
    //draw panes for indicating measurement
    for(idx=[0 : len(measuringfs) - 1]){
            if(measuringfs[idx] == true){
                drawMeasurementPane(idx + 1, 0);
            }
    }
}


/// Beschreibung: Erstellt eine Visualisierung von 3 Qbit und bearbeitet diese je nach Modus und Parametern
module visualize3Qbit(mode, probs = [], measuringfs = [], measured = [], measureResults = [] ){
    
    //Draw Probability Background Squares
    draw3QbitVertices();
	draw3QbitProbabilities(probs);
	draw3QbitStateLabels(3);
	draw3QbitProbLabels(probs);
    
    
    if(mode == 3){
        
        //Build Square with edges, cut edges if indicating measurement
        difference(){
            union(){draw3QbitEdges();}
            for(idx=[0 : len(measuringfs) - 1]){
                if(measuringfs[idx] == true){
                    drawMeasurementPane(idx + 1, 0);
                }
            }
        }
        
        //draw panes for indicating measurement
        for(idx=[0 : len(measuringfs) - 1]){
            if(measuringfs[idx] == true){
                drawMeasurementPane(idx + 1, 0);
            }
        }
    }
}





// Zeichnet die orangenen Hintergründe für die Darstellung von 1 Qbit
module draw1QbitVertices(){
    drawSquare([-t,-t,-t],sq,true,"orange"); //0
    drawSquare([t,-t,-t],sq,true,"orange");  //1
}

// Zeichnet die orangenen Hintergründe für die Darstellung von 2 Qbit
module draw2QbitVertices(){
    draw1QbitVertices();
    drawSquare([-t,-t,t],sq,true,"orange");  //01
    drawSquare([t,-t,t],sq,true,"orange");   //11
}

// Zeichnet die orangenen Hintergründe für die Darstellung von 3 Qbit
module draw3QbitVertices(){
    draw2QbitVertices();
    drawSquare([-t,t,-t],sq,true,"orange");  //001
    drawSquare([-t,t,t],sq,true,"orange");   //011
    drawSquare([t,t,-t],sq,true,"orange");   //101
    drawSquare([t,t,t],sq,true,"orange");    //111
}

// Zeichnet die färbenden Quadrate für die Darstellung von 1 Qbit
module draw1QbitProbabilities(WSK=[]){
    drawSquare([-t-dp_h,-t,-t-dp_h],[abs(WSK[0])*b,abs(WSK[0])*b,d+dp_u],false,WSK[0] < 0 ? colorR:colorWskPositive);   //0 
    drawSquare([t-dp_h,-t,-t-dp_h],[abs(WSK[1])*b,abs(WSK[1])*b,d+dp_u],false,WSK[1] < 0 ? colorR:colorWskPositive);    //1
}
// Zeichnet die färbenden Quadrate für die Darstellung von 2 Qbit
module draw2QbitProbabilities(WSK=[]){
	drawSquare([-t-dp_h,-t,-t-dp_h],[abs(WSK[0])*b,abs(WSK[0])*b,d+dp_u],false,WSK[0] < 0 ? colorR:colorWskPositive);   //00 
    drawSquare([t-dp_h,-t,-t-dp_h],[abs(WSK[2])*b,abs(WSK[2])*b,d+dp_u],false,WSK[2] < 0 ? colorR:colorWskPositive);    //10
    drawSquare([-t-dp_h,-t,t-dp_h],[abs(WSK[1])*b,abs(WSK[1])*b,d+dp_u],false,WSK[1] < 0 ? colorR:colorWskPositive);    //01
    drawSquare([t-dp_h,-t,t-dp_h],[abs(WSK[3])*b,abs(WSK[3])*b,d+dp_u],false,WSK[3] < 0 ? colorR:colorWskPositive);     //11 
}
// Zeichnet die färbenden Quadrate für die Darstellung von 3 Qbit
module draw3QbitProbabilities(WSK=[]){
    drawSquare([-t-dp_h,-t,-t-dp_h],[abs(WSK[0])*b,abs(WSK[0])*b,d+dp_u],false,WSK[0] < 0 ? colorR:colorWskPositive);   //000 
    drawSquare([t-dp_h,-t,-t-dp_h],[abs(WSK[4])*b,abs(WSK[4])*b,d+dp_u],false,WSK[4] < 0 ? colorR:colorWskPositive);    //100
    drawSquare([-t-dp_h,-t,t-dp_h],[abs(WSK[2])*b,abs(WSK[2])*b,d+dp_u],false,WSK[2] < 0 ? colorR:colorWskPositive);    //010
    drawSquare([t-dp_h,-t,t-dp_h],[abs(WSK[6])*b,abs(WSK[6])*b,d+dp_u],false,WSK[6] < 0 ? colorR:colorWskPositive);     //110
	
    drawSquare([-t-dp_h,t,-t-dp_h],[abs(WSK[1])*b,abs(WSK[1])*b,d+dp_u],false,WSK[1] < 0 ? colorR:colorWskPositive);    //001 
    drawSquare([-t-dp_h,t,t-dp_h],[abs(WSK[3])*b,abs(WSK[3])*b,d+dp_u],false,WSK[3] < 0 ? colorR:colorWskPositive);     //011 
    drawSquare([t-dp_h,t,-t-dp_h],[abs(WSK[5])*b,abs(WSK[5])*b,d+dp_u],false,WSK[5] < 0 ? colorR:colorWskPositive);     //101 
    drawSquare([t-dp_h,t,t-dp_h],[abs(WSK[7])*b,abs(WSK[7])*b,d+dp_u],false,WSK[7] < 0 ? colorR:colorWskPositive);      //111  
}

//Zeichnet eine Quadratfläche
module drawSquare(trans,size,center,color){
    translate(trans) rotate([90,0,0]) color(color)
    cube(size,center);
}


//Zeichnet die State Label für 1 Qbit
module draw1QbitStateLabels(digits){
     drawStateLabel(digits==1?"|0>":digits==2?"|00>":"|000>", [0, 0, 0]);
     drawStateLabel(digits==1?"|1>":digits==2?"|10>":"|100>", [h, 0, 0]);
}
//Zeichnet die State Label für 2 Qbit
module draw2QbitStateLabels(digits){
    draw1QbitStateLabels(digits);
    drawStateLabel(digits==3?"|010>" : "|01>", [0, 0, h]);
    drawStateLabel(digits==3?"|110>" : "|11>", [h, 0, h]);
}
//Zeichnet die State Label für 3 Qbit
module draw3QbitStateLabels(digits){
    draw2QbitStateLabels(digits);
    drawStateLabel("|001>", [0, h, 0]);
	drawStateLabel("|011>", [0, h, h]);
	drawStateLabel("|101>", [h, h, 0]);
	drawStateLabel("|111>", [h, h, h]);
}


//Basisfunktion zum zeichnen eines State Labels
module drawStateLabel(text, offsetVector = []){
    translate(state_pos_vector + offsetVector) rotate([90,0,0]) color(state_color) linear_extrude(height = state_height){
	text(text, size = state_size);
	}
}
//Basisfunktion zum zeichnen eines färbenden Quadrats
module drawProbLabel(text, offsetVector = []){
    translate(prob_pos_vector + offsetVector) rotate([90,0,0]) color(prob_color) linear_extrude(height = prob_height){
	text(text, size = prob_size);
	}
}

//Zeichnet die Prob Labels für 1 QBit
module draw1QbitProbLabels(WSK=[]){
    drawProbLabel(str(WSK[0]), [0, 0, 0]); //0
    drawProbLabel(str(WSK[1]), [h, 0, 0]); //1
}
//Zeichnet die Prob Labels für 2 QBit
module draw2QbitProbLabels(WSK=[]){
    draw1QbitProbLabels(WSK);
    drawProbLabel(str(WSK[2]), [0, 0, h]);    //01
    drawProbLabel(str(WSK[3]), [h, 0, h]);//11
}
//Zeichnet die Prob Labels für 3 QBit
module draw3QbitProbLabels(WSK=[]){
    draw2QbitProbLabels(WSK);
    drawProbLabel(str(WSK[4]), [0, h, 0]);
	drawProbLabel(str(WSK[5]), [0, h, h]);
	drawProbLabel(str(WSK[6]), [h, h, 0]);
	drawProbLabel(str(WSK[7]), [h, h, h]);
}

//Zeichnet die Kanten für 1 Qbit
module draw1QbitEdges(){
    drawCylinder([t,-t,-t],[0,-90,0],false);
}
//Zeichnet die Kanten für 2 Qbit
module draw2QbitEdges(){
    draw1QbitEdges();
    drawCylinder([-t,-t,0],[0,0,0],true);
    drawCylinder([t,-t,0],[0,0,0],true);
    drawCylinder([t,-t,t],[0,-90,0],false);
}
//Zeichnet die Kanten für 3 Qbit
module draw3QbitEdges(){
    draw2QbitEdges();
    drawCylinder([-t,t,0],[0,0,0],true);
    drawCylinder([t,t,0],[0,0,0],true);
    drawCylinder([-t,-t,t],[-90,0,0],false);
    drawCylinder([-t,t,t],[0,90,0],false);
    drawCylinder([t,t,t],[90,0,0],false);
    drawCylinder([-t,-t,-t],[-90,0,0],false);
    drawCylinder([-t,t,-t],[0,90,0],false);
    drawCylinder([t,t,-t],[90,0,0],false);
}

//Basisfunktion zum zeichnen einer Kante
module drawCylinder(trans,rot,center){
//Zeichnet einen Zylinder mit abgeru
    translate(trans) rotate(rot) color(colorO,edgeOpacity)
    cylinder(h=h,d=d,$fn=fn,center=center);
    translate(trans) sphere(r=r,$fn=fn);
}

//Zeichnet Measurement Panes ein, die eine Messung an einer Achse anzeigen
module drawMeasurementPane(targetQubit, size){
    color(colorB, opacity) 
    translate(size == 1 ? [0,-h_sp/3,-h_sp/3] : [0,0,0])
    cube(size == 1 ? [d_sp,h_sp/3,h_sp/3]:[targetQubit==1?d_sp:h_sp,targetQubit==3?d_sp:h_sp,targetQubit==2?d_sp:h_sp], center=true);
}
